use codespan_reporting::files::SimpleFiles;
use codespan_reporting::term;
use codespan_reporting::term::termcolor::{ColorChoice, StandardStream};
use deimos::ast::build::build_ast;
use deimos::name_analysis::analyze_names;
use deimos::name_analysis::symbol_table::SymbolTable;
use deimos::parser::{DeimosParser, Rule};
use pest::Parser;
use std::path::Path;

pub fn name_analysis(path: &Path) -> Result<(), Box<dyn std::error::Error>> {
    let src = std::fs::read_to_string(path).unwrap();
    let parse_result = DeimosParser::parse(Rule::CompilationUnit, &src);
    let mut files = SimpleFiles::new();
    let file_id = files.add(path.display().to_string(), &src);

    match parse_result {
        Ok(mut pairs) => {
            let compilation_unit_pair = pairs.next().unwrap();
            let mut compilation_unit = build_ast(file_id, compilation_unit_pair);
            let mut symbol_table = SymbolTable::new();
            let diagnostics = analyze_names(file_id, &mut compilation_unit, &mut symbol_table);
            if diagnostics.is_empty() {
                println!("Name analysis complete.");
                println!("Symbol table\n-------\n{}", symbol_table);
                Ok(())
            } else {
                let writer = StandardStream::stderr(ColorChoice::Always);
                let config = term::Config::default();
                for diagnostic in diagnostics {
                    term::emit(&mut writer.lock(), &config, &files, &diagnostic)?;
                }
                Ok(())
            }
        }
        Err(e) => Err(e.into()),
    }
}
